@@ -2,10 +2,7 @@
 """
 bamprep.py - prepare SAM/BAM file for phase change detection
 
-TODO: add explicit check for samtools in userpath
--> raise informative error if not present
-
-this script will automatically try to detect your samtools binary by default - 
+this script will automatically try to detect your samtools binary by default -
 the one it uses can be found by running `which samtools` in the command line
 (alternatively, you could symlink your preferred version of samtools into a bin/ folder
 in your project dir to keep paths clean, and then provide `./bin/samtools`
@@ -14,14 +11,19 @@ for the path - recommended)
 
 import os
 import re
+import sys
 import argparse
 import subprocess
-import pysam
-from tqdm import tqdm
+
+class ReadcombParser(argparse.ArgumentParser):
+    def error(self, message):
+        sys.stderr.write(f'error: {message}\n')
+        self.print_help()
+        sys.exit(1)
 
 def arg_parser():
-    parser = argparse.ArgumentParser(
-        description='prep SAM/BAM for phase change detection', 
+    parser = ReadcombParser(
+        description='prep SAM/BAM for phase change detection',
         usage='readcomb-bamprep [options]')
 
     parser.add_argument('-b', '--bam', required=True,
@@ -37,128 +39,120 @@ def arg_parser():
         bars in filtering step).')
     parser.add_argument('-o', '--outdir', required=False,
         type=str, help='Directory to write to [optional] (default: current dir)')
+    parser.add_argument('--version', action='version', version='readcomb 0.0.8')
 
-    args = parser.parse_args()
+    return parser
 
-    return args.bam, args.samtools, args.threads, args.index_csi, \
-        args.no_progress, args.outdir
-
-def bamprep(fname, samtools, threads, index_csi, no_progress, outdir):
+def bamprep(args):
     """
     Prepare BAM for phase change detection with readcomb.
-    
+
     Script will:
     1. Apply filters (proper pair, not secondary, not supplementary)
     2. Sort by read name instead of position (samtools sort)
     3. Create .bai file (samtools index)
 
     The output file will be named similarly to the input - only
-    the extension will have been changed. 
-    
+    the extension will have been changed.
+
     Parameters
     ----------
-    fname : str
-        path to input SAM/BAM
-    outdir : str
-        path to directory to write to (optional)
-
-    Returns
-    -------
-    None
+    args : Namespace
+        Namespace containing all user given arguements compiled by arg_parse()
     """
     # check extension
-    match = re.search('\.[sb]am$', fname)
+    match = re.search(r'\.[sb]am$', args.bam)
     if match:
         extension = match.group()
-        basename = fname[:fname.find(extension)]
+        basename = os.path.basename(args.bam[:args.bam.find(extension)])
     else:
         raise ValueError('input file is not SAM or BAM - make sure extension is correct')
 
     # get samtools path if not specified
-    if not samtools:
+    if not args.samtools:
         samtools_check_cmd = 'which samtools'
         proc = subprocess.Popen(samtools_check_cmd.split(' '),
                 stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        out, err = proc.communicate()
+        out, _ = proc.communicate()
         if not out:
             raise RuntimeError("""ERROR: samtools not found. Please
                     either ensure samtools is in your PATH or specify a
                     samtools binary using the --samtools argument""")
         else:
-            samtools = out.decode('utf-8').rstrip('\n')
+            args.samtools = out.decode('utf-8').rstrip('\n')
 
     # format args
-    if outdir:
-        if not outdir.endswith('/'):
-            outdir += '/'
-    elif not outdir:
-        outdir = ''
+    if args.outdir:
+        if not args.outdir.endswith('/'):
+            args.outdir += '/'
+    elif not args.outdir:
+        args.outdir = './'
         print('[readcomb] no outdir specified, saving to current working directory')
 
     # filter out non proper pair, supplementary, and secondary reads
-    print('[readcomb] Filtering...') 
-    filter_pos_cmd = '{s} view -O bam '.format(s=samtools)
+    print('[readcomb] Filtering...')
+    filter_pos_cmd = f'{args.samtools} view -O bam '
     filter_pos_cmd += '-f 0x2 -F 0x100 -F 0x800 '
-    filter_pos_cmd += '-o {base}.temp.filtered.bam {fname}'.format(
-            s=samtools, out=outdir, base=basename, fname=fname)
-    proc = subprocess.run(filter_pos_cmd.split(' '))
-    proc.check_returncode() # raises CalledProcessError if proc failed
+    filter_pos_cmd += f'-o {args.outdir}{basename}.temp.filtered.bam {args.bam}'
+    print(f'[readcomb] cmd: {filter_pos_cmd}')
+    proc = subprocess.run(filter_pos_cmd.split(' '), check=True)
 
-    if not no_progress:
+    if not args.no_progress:
     # sort positionally for bai file creation
         print('[readcomb] Positionally sorting for bai creation...')
-        sort_pos_cmd = '{s} sort -O bam '.format(s=samtools)
-        if threads:
-            sort_pos_cmd += '-@{n} '.format(n=threads)
-        sort_pos_cmd += '-o {base}.sorted.bam {base}.temp.filtered.bam'.format(base=basename)
-        proc = subprocess.run(sort_pos_cmd.split(' '))
-        proc.check_returncode()
+        sort_pos_cmd = f'{args.samtools} sort -O bam '
+        if args.threads:
+            sort_pos_cmd += f'-@{args.threads} '
+        sort_pos_cmd += f'-o {args.outdir}{basename}.sorted.bam '
+        sort_pos_cmd += f'{args.outdir}{basename}.temp.filtered.bam'
+        print('[readcomb] Sorting positionally for .bai creation...')
+        print(f'[readcomb] cmd: {sort_pos_cmd}')
+        proc = subprocess.run(sort_pos_cmd.split(' '), check=True)
 
         # create bai file
-        if index_csi:
+        if args.index_csi:
             idx_type = '.csi'
         else:
             idx_type = '.bai'
-        print('[readcomb] Creating {} index file...'.format(idx_type))
-        index_cmd = '{s} index {base}.sorted.bam {out}{base}.sorted{idx}'.format(
-                s=samtools, out=outdir, base=basename, idx=idx_type)
+        print(f'[readcomb] Creating {idx_type} index file...')
+        index_cmd = f'{args.samtools} index {args.outdir}{basename}.sorted.bam '
+        index_cmd += f'{args.outdir}{basename}.sorted{idx_type}'
+        print(f'[readcomb] cmd: {index_cmd}')
         proc = subprocess.run(index_cmd.split(' '))
-        proc.check_returncode()
 
         # remove positionally sorted file
         try:
-            os.remove('{base}.sorted.bam'.format(base=basename))
+            os.remove(f'{basename}.sorted.bam')
         except OSError as e:
-            raise FileNotFoundError('Unable to delete {} - {}'.format(e.filename, e.strerror))
+            raise FileNotFoundError(f'Unable to delete {e.filename} - {e.strerror}') from e
 
     # sort by name and ensure file is in BAM format
     print('[readcomb] Creating read name sorted file...')
-    sort_cmd = '{s} sort -n -O bam '.format(s=samtools)
-    if threads:
-        sort_cmd += '-@{n} '.format(n=threads)
-    sort_cmd += '-o {out}{base}.sorted.bam {base}.temp.filtered.bam'.format(
-            out=outdir, base=basename)
-    proc = subprocess.run(sort_cmd.split(' '))
-    proc.check_returncode() 
+    sort_cmd = f'{args.samtools} sort -n -O bam '
+    if args.threads:
+        sort_cmd += f'-@{args.threads} '
+    sort_cmd += f'-o {args.outdir}{basename}.sorted.bam {args.outdir}{basename}.temp.filtered.bam'
+    print(f'[readcomb] cmd: {sort_cmd}')
+    proc = subprocess.run(sort_cmd.split(' '), check=True)
 
     # remove filtered unsorted file
     print('[readcomb] Removing temp files...')
     try:
-        os.remove('{base}.temp.filtered.bam'.format(base=basename))
+        os.remove(f'{args.outdir}{basename}.temp.filtered.bam')
     except OSError as e:
-        raise FileNotFoundError('Unable to delete {} - {}'.format(e.filename, e.strerror))
-
+        raise FileNotFoundError(f'Unable to delete {e.filename} - {e.strerror}')
 
 def main():
-    bam, samtools, threads, index_csi, no_progress, outdir = arg_parser()
-    if samtools:
-        if not os.path.isfile(samtools):
-            raise ValueError('ERROR: No samtools binary found at {}'.format(samtools))
-    bamprep(bam, samtools, threads, index_csi, no_progress, outdir)
+    parser = arg_parser()
+    # Namespace of arguments
+    args = parser.parse_args()
+
+    if args.samtools:
+        if not os.path.isfile(args.samtools):
+            raise ValueError(f'ERROR: No samtools binary found at {args.samtools}')
+    bamprep(args)
     print('[readcomb] Complete.')
 
+
 if __name__ == '__main__':
     main()
-
-        
-
