@@ -7,49 +7,45 @@ import sys
 import argparse
 import time
 import datetime
-import pysam
 import subprocess
-import pandas as pd
 from io import BytesIO
 from multiprocessing import Queue
 from multiprocessing import Process
+
+import pysam
+import pandas as pd
 from cyvcf2 import VCF
 from tqdm import tqdm
 
+class ReadcombParser(argparse.ArgumentParser):
+    def error(self, message):
+        sys.stderr.write(f'error: {message}\n')
+        self.print_help()
+        sys.exit(1)
+
+
 def arg_parser():
-    parser = argparse.ArgumentParser(
+    parser = ReadcombParser(
         description='filter BAM for reads containing phase changes',
         usage='readcomb-filter [options]')
 
     parser.add_argument('-b', '--bam', required=True,
-                        type=str, help='BAM to filter, required')
-
+        type=str, help='BAM to filter, required')
     parser.add_argument('-v', '--vcf', required=True,
-                        type=str, help='VCF containing parents, required')
-
+        type=str, help='VCF containing parents, required')
     parser.add_argument('-p', '--processes', required=False, type=int, default=4,
-                        help='Number of processes to run readcomb filter on, default is 1')
-
+        help='Number of processes to run readcomb filter on, default is 1')
     parser.add_argument('-m', '--mode', required=False, type=str, default='phase_change',
-                        help='Read filtering mode (default phase_change) \
-                             [phase_change|no_match]')
-
-    parser.add_argument('-l', '--log', required=False,
-                        type=str, help='Filename for log metric output [optional]')
-
+        help='Read filtering mode (default phase_change) [phase_change|no_match]')
+    parser.add_argument('-l', '--log', required=False, default=None,
+        type=str, help='Filename for log metric output [optional]')
     parser.add_argument('-o', '--out', required=False, type=str, default='recomb_diagnosis',
-                        help='File to write to (default recomb_diagnosis)')
+        help='File to write to (default recomb_diagnosis)')
+    parser.add_argument('-q', '--quality', required=False, type=int, default=30,
+        help='Filter quality for individual bases in a sequence, default is 30')
+    parser.add_argument('--version', action='version', version='readcomb 0.0.8')
 
-    args = parser.parse_args()
-
-    return {
-        'bam': args.bam,
-        'vcf': args.vcf,
-        'processes': args.processes,
-        'mode': args.mode,
-        'log': args.log,
-        'out': args.out,
-        }
+    return parser
 
 class SilentVCF:
     # helper class for check_variants that silences cyvcf2 warnings
@@ -63,7 +59,7 @@ class SilentVCF:
 
 def check_variants(vcf_file_obj, chromosome, left_bound, right_bound):
     """
-    Fetch variants from vcf file in given bounds.
+    Fetch variants from VCF file in given bounds.
 
     Parameters
     ----------
@@ -83,8 +79,7 @@ def check_variants(vcf_file_obj, chromosome, left_bound, right_bound):
 
     # 1 is added to record.reference_start and the following parameter because vcf is 1 indexed
     # in order to keep code consistent
-    region = '{c}:{l}-{r}'.format(c=chromosome, l=left_bound+1, r=right_bound+1)
-    
+    region = f'{chromosome}:{left_bound+1}-{right_bound}'
     with SilentVCF():
         records = [rec for rec in vcf_file_obj(region)]
     return records
@@ -93,8 +88,8 @@ def cigar(record):
     """
     Realign the given query segment to its reference.
 
-    Rebuild the sequence using the query sequence and cigar tuple from the bam record to realign
-    it with the reference sequence prior to variant matching
+    Rebuild the sequence using the query sequence and cigar tuple from the 
+    bam record to realign it with the reference sequence prior to variant matching
 
     Parameters
     ----------
@@ -157,13 +152,13 @@ def cigar(record):
 
     return ''.join(segment)
 
-def phase_detection(variants, segment, record):
+def phase_detection(variants, segment, record, args):
     """
     Detect haplotype of variants in the given DNA sequence.
 
-    Takes a ``segment`` and a list of ``cyvcf2`` variants in the region of the segment 
-    to detect the haplotype of the sequence and generate a list of {'1','2','N'} to represent it
-    in the order of the variants on the segment
+    Takes a ``segment`` and a list of ``cyvcf2`` variants in the region of the segment
+    to detect the haplotype of the sequence and generate a list of {'1','2','N'} to
+    represent it in the order of the variants on the segment
 
     Parameters
     ----------
@@ -173,6 +168,8 @@ def phase_detection(variants, segment, record):
         realigned sequence built by ``cigar()``
     record : pysam.AlignedSegment
         bam record from pysam alignment file
+    args : Namespace
+        Namespace containing all user given arguments compiled by arg_parse()
 
     Returns
     -------
@@ -199,16 +196,31 @@ def phase_detection(variants, segment, record):
         # ignore uninformative variants
         if parent1 == parent2:
             continue
-        
+
         # ignore if variant is before sequence
         if idx < 0:
             continue
-        
-        if variant.is_indel:
-            # check if indel is outside of segment
-            if idx + max(len(parent1), len(parent2)) > len(segment):
-                continue
 
+        # check if variant is outside of segment
+        if idx + max(len(parent1), len(parent2)) > len(segment):
+            continue
+
+        # ignore variant if quality of sequencing at that base is below threshold
+        query_qualities = record.query_qualities.tolist()
+        # realign query_qualities if there are insertions
+        count = 0
+        insertions = 0
+        for tupl in cigar_tuples:
+            if count > idx:
+                break
+            if tupl[0] == 2:
+                insertions += tupl[1]
+            count += tupl[1]
+
+        if query_qualities[idx - insertions] < args.quality:
+            continue
+
+        if variant.is_indel:
             parent1_match = segment[idx:idx + len(parent1)] == parent1
             parent2_match = segment[idx:idx + len(parent2)] == parent2
 
@@ -231,9 +243,6 @@ def phase_detection(variants, segment, record):
                     variant_lst.append('N')
 
         else: # variant is a SNP
-            if idx >= len(segment):
-                break
-
             if segment[idx] == parent1:
                 variant_lst.append('1')
 
@@ -246,17 +255,18 @@ def phase_detection(variants, segment, record):
     return variant_lst
 
 class Processor(Process):
-    def __init__(self, input_queue, counter_queue, 
+    def __init__(self, input_queue, counter_queue,
                 writer_queue, args, **kwargs):
         """
         Initialize the read pair processor.
 
-        Processor is inherited from ``multiprocessing.Process``; when started it receives pairs of
-        bam sequences through the ``input_queue`` from the scheduler and does phase change analysis, outputting 
-        through ``writer_queue`` bams that fit the user given arguement criteria to ``Writer``.
-        
-        When the scheduler has parsed and distributed all bam sequences, the ``Processor`` receives
-        a ``None`` through the ``input_queue`` and ends its loop and prepares to join.
+        Processor is inherited from ``multiprocessing.Process``; when started it receives
+        pairs of bam sequences through the ``input_queue`` from the scheduler and does
+        phase change analysis, outputting through ``writer_queue`` bams that fit the user
+        given argument criteria to ``Writer``.
+
+        When the scheduler has parsed and distributed all bam sequences, the ``Processor``
+        receives a ``None`` through the ``input_queue`` and ends its loop and prepares to join.
 
         Parameters
         ----------
@@ -266,18 +276,18 @@ class Processor(Process):
             output queue for result of phase change analysis
         writer_queue : multiprocessing.Queue
             output queue for bam read pairs in string form that fulfill the user given criteria
-        args : dictionary of strings
-            dictionary containing all user given arguements compiled by arg_parse()
-        **kwargs 
+        args : Namespace
+            Namespace containing all user given arguments compiled by arg_parse()
+        **kwargs
             extra parameters passed onto super class ``multiprocessing.Queue()``
         """
         Process.__init__(self, **kwargs)
         self.input_queue = input_queue
         self.counter_queue = counter_queue
         self.writer_queue = writer_queue
-        self.vcf_file_obj = VCF(args['vcf'])
+        self.vcf_file_obj = VCF(args.vcf)
         # header used for converting bam string to bam object
-        self.header = pysam.AlignmentFile(args['bam'], 'r').header
+        self.header = pysam.AlignmentFile(args.bam, 'r').header
         self.args = args
 
     def run(self):
@@ -285,9 +295,9 @@ class Processor(Process):
         Start the ``Processor``
 
         ``run()`` is automatically called when the scheduler runs ``Processor.start()``,
-        this begins the operations of both the ``multiprocessing.Process()`` super class and
-        ``Processor``. 
-        
+        this begins the operations of both the ``multiprocessing.Process()`` super 
+        class and ``Processor``.
+
         See ``check_variants()``, ``cigar()``, and ``phase_detection`` for
         more information on the analysis process.
         """
@@ -298,15 +308,15 @@ class Processor(Process):
             record_1 = pysam.AlignedSegment.fromstring(pair[0], self.header)
             record_2 = pysam.AlignedSegment.fromstring(pair[1], self.header)
 
-            variants_1 = check_variants(self.vcf_file_obj, record_1.reference_name,
-                                    record_1.reference_start,
-                                    record_1.reference_start + record_1.query_alignment_length)
-            variants_2 = check_variants(self.vcf_file_obj, record_2.reference_name,
-                                    record_2.reference_start,
-                                    record_2.reference_start + record_2.query_alignment_length)
+            variants_1 = check_variants(
+                self.vcf_file_obj, record_1.reference_name, record_1.reference_start, 
+                record_1.reference_start + record_1.query_alignment_length)
+            variants_2 = check_variants(
+                self.vcf_file_obj, record_2.reference_name, record_2.reference_start, 
+                record_2.reference_start + record_2.query_alignment_length)
 
             self.counter_queue.put('seq')
-            if len(variants_1) + len(variants_2) > 1: 
+            if len(variants_1) + len(variants_2) > 1:
                 self.counter_queue.put('seq_with_variants')
             # skip if not enough variants
             else:
@@ -316,24 +326,23 @@ class Processor(Process):
             segment_1 = cigar(record_1)
             segment_2 = cigar(record_2)
 
-            variant_lst = phase_detection(variants_1, segment_1, record_1) + phase_detection(variants_2, segment_2, record_2)
+            variant_lst = phase_detection(variants_1, segment_1, record_1, self.args)
+            variant_lst += phase_detection(variants_2, segment_2, record_2, self.args)
 
             if '1' in variant_lst and '2' in variant_lst:
                 self.counter_queue.put('phase_change')
 
-                if 'phase_change' in self.args['mode']:
+                if 'phase_change' in self.args.mode:
                     self.writer_queue.put(pair)
-                
+
             if 'N' in variant_lst:
                 self.counter_queue.put('no_match')
 
-                if 'no_match' in self.args['mode']:
+                if 'no_match' in self.args.mode:
                     self.writer_queue.put(pair)
 
             pair = self.input_queue.get(block=True)
 
-
-    
 class Counter(Process):
     def __init__(self, input_queue, args, **kwargs):
         """
@@ -341,17 +350,17 @@ class Counter(Process):
 
         Inherited from ``multiprocessing.Process``, ``Counter`` receives sequence information
         from Processor processes through ``input_queue`` and tallies them to generate a ``tqdm``
-        progress bar in ``STDOUT``. 
-        
+        progress bar in ``STDOUT``.
+
         After being passed a ``None`` from the ``Scheduler, ``Counter``
-        either writes tallied results to ``STDOUT`` or a log depending on the user's arguements.
-        
+        either writes tallied results to ``STDOUT`` or a log depending on the user's arguments.
+
         Parameters
         ----------
         input_queue : multiprocessing.Queue
             get sequence information from all ``Processor`` processes
-        args : dictionary of str
-            dictionary containing all user given arguements compiled by arg_parse()
+        args : Namespace
+            Namespace containing all user given arguments compiled by arg_parse()
         **kwargs
             extra args that are passed onto super class ``Process``
         """
@@ -364,6 +373,7 @@ class Counter(Process):
             'seq': 0
             }
         self.args = args
+        self.progress = None
 
     def run(self):
         """
@@ -378,8 +388,8 @@ class Counter(Process):
         start = time.time()
 
         # create tqdm iteration counter if no bars
-        if 'pair_count' in self.args:
-            self.progress = tqdm(total=self.args['pair_count'])
+        if self.args.pair_count:
+            self.progress = tqdm(total=self.args.pair_count)
         else:
             self.progress = tqdm()
 
@@ -390,9 +400,9 @@ class Counter(Process):
             # update iteration counter if sequence
             if count == 'seq':
                 self.progress.update(n=1)
-            
+
             count = self.input_queue.get(block=True)
-        
+
         self.progress.close()
 
         # end timer
@@ -406,36 +416,38 @@ class Counter(Process):
         print('{} reads did not have enough variants (> 0) to call'.format(
             self.counters['seq'] - self.counters['seq_with_variants']
         ))
-        print('time taken: {}'.format(runtime))
-        
-        # write to log if argument defined
-        if self.args["log"]:
-            # determine if we are adding to file or creating new log file
-            needs_header = False if os.path.isfile(self.args['log']) else True
+        print(f'time taken: {runtime}'.format(runtime))
 
-            with open(self.args["log"], 'a') as f:
+        # write to log if argument defined
+        if self.args.log:
+            # determine if we are adding to file or creating new log file
+            needs_header = bool(not os.path.isfile(self.args.log))
+
+            # convert argparse namespace to dictionary to be more easily manipulated
+            args_dict = vars(self.args)
+
+            with open(args_dict, 'a') as f:
                 if needs_header:
                     fieldnames = ['time',
                                 'phase_change_reads',
-                                'no_match_reads', 
-                                'seq_with_variants', 
-                                'no_variant_reads', 
+                                'no_match_reads',
+                                'seq_with_variants',
+                                'no_variant_reads',
                                 'seqs',
                                 'time_taken']
-                    fieldnames.extend(sorted(self.args.keys()))
+                    fieldnames.extend(sorted(args_dict.keys()))
                     f.write(','.join(fieldnames) + '\n')
 
-                out_values = [datetime.datetime.now(), 
-                            self.counters['phase_change'], 
+                out_values = [datetime.datetime.now(),
+                            self.counters['phase_change'],
                             self.counters['no_match'],
-                            self.counters['seq_with_variants'], 
+                            self.counters['seq_with_variants'],
                             self.counters['seq'] - self.counters['seq_with_variants'],
                             self.counters["seq"],
                             runtime]
-                out_values.extend([self.args[key] for key in sorted(self.args.keys())])
+                out_values.extend([args_dict[key] for key in sorted(args_dict.keys())])
                 f.write(','.join([str(n) for n in out_values]) + '\n')
 
-
 class Writer(Process):
     def __init__(self, input_queue, args, **kwargs):
         """
@@ -446,24 +458,24 @@ class Writer(Process):
         ``Writer`` converts bam records in string form to ``pysam.AlignedSegment`` objects and
         writes them to a SAM file. When ``Writer`` receives a ``None`` from the Scheduler, the
         writing loop is ended and the output file is closed.
-        
+
 
         Parameters
         ----------
         input_queue : multiprocessing.Queue
             queue to receive bam record strings from all ``Processor`` processes
-        args : dictionary of str
-            dictionary containing all user given arguements compiled by arg_parse()
+        args : Namespace
+            Namespace containing all user given arguments compiled by arg_parse()
         **kwargs
             extra args that are passed onto super class ``Process``
         """
         Process.__init__(self, **kwargs)
         self.input_queue = input_queue
 
-        pysam_obj = pysam.AlignmentFile(args['bam'], 'r')
-        self.out = pysam.AlignmentFile(args['out'] + '.sam', 'wh', 
-                                        template=pysam_obj)
-        self.header = pysam.AlignmentFile(args['bam'], 'r').header
+        pysam_obj = pysam.AlignmentFile(args.bam, 'r')
+        self.out = pysam.AlignmentFile(
+            args.out + '.sam', 'wh', template=pysam_obj)
+        self.header = pysam.AlignmentFile(args.bam, 'r').header
 
     def run(self):
         """
@@ -485,40 +497,40 @@ class Writer(Process):
         self.out.close()
 
 
-
-
 def matepair_process():
     """
     Start processes, queues, and distribute bam records to ``Processors``
 
     ``matepair_process`` calls ``arg_parse()`` to parse user given arguments.
 
-    It then initializes and starts ``Processors``, ``Counter``, and ``Writer`` processes/daemons 
-    and then equally distribute the bam records in string form to ``Processors``. 
-    
-    After all bam records have been distributed, ``None`` is passed to ``Processor`` 
+    It then initializes and starts ``Processors``, ``Counter``, and ``Writer``
+    processes/daemons and then equally distribute the bam records
+    in string form to ``Processors``.
+
+    After all bam records have been distributed, ``None`` is passed to ``Processor``
     processes, ``Counter process, and ``Writer`` process to signal their stop and
     ``close()`` the processes.
     """
-    # dictionary of arguements
-    args = arg_parser()
+    parser = arg_parser()
+    args = parser.parse_args()
 
     # idxstats on bam/bai file
-    if not os.path.isfile(args['bam'] + '.bai'):
+    if not os.path.isfile(args.bam + '.bai'):
         print('Bai file not found, continuing without progress bars')
+        args.pair_count = None
     else:
-        stats = subprocess.check_output(['samtools', 'idxstats', args['bam']])
-        stats_table = pd.read_csv(BytesIO(stats), sep='\t', 
-                                    names=['chrom', 'length', 'map_reads', 'unmap_reads'])
+        stats = subprocess.check_output(['samtools', 'idxstats', args.bam])
+        stats_table = pd.read_csv(
+            BytesIO(stats), sep='\t',
+            names=['chrom', 'length', 'map_reads', 'unmap_reads'])
         pairs_sum = stats_table['map_reads'].sum()
-        args['pair_count'] = int(pairs_sum / 2) # divide 2 to get pairs
+        args.pair_count = int(pairs_sum / 2) # divide 2 to get pairs
 
         if pairs_sum % 2 != 0:
             raise ValueError('Preprocessing of bam file went wrong')
 
-
     # pysam argument object
-    bam = pysam.AlignmentFile(args['bam'], 'r')
+    bam = pysam.AlignmentFile(args.bam, 'r')
 
     print('Creating processes')
     # set up counter and writer
@@ -534,12 +546,12 @@ def matepair_process():
     processes = []
     input_queues = []
     # set up processes
-    for i in range(args['processes']):
+    for i in range(args.processes):
         input_queue = Queue()
         input_queues.append(input_queue)
-        processes.append(Processor(input_queue, count_input, 
-                            write_input, args, 
-                            daemon=True, name='Process ' + str(i))) 
+        processes.append(
+            Processor(input_queue, count_input, write_input, args,
+                daemon=True, name='Process ' + str(i)))
         processes[i].start()
 
     prev_record = None
@@ -557,23 +569,23 @@ def matepair_process():
         else:
             if prev_record.query_name != record.query_name:
                 raise ValueError('Preprocessing of bam file went wrong')
-            
+
             # convert bam sequence to string so it can be pickled and sent in queue
             pair = [prev_record.to_string(), record.to_string()]
 
             input_queues[process_idx].put(pair)
 
             # give record to next process
-            if process_idx < args['processes'] - 1:
+            if process_idx < args.processes - 1:
                 process_idx += 1
             else:
-                process_idx = 0 
+                process_idx = 0
 
             # clear the pair
             prev_record = None
 
     # shut down processes
-    for i in range(len(processes)):
+    for i, _ in enumerate(processes):
         # pass None to let processes know all bam sequences have been scheduled
         input_queues[i].put(None)
         input_queues[i].close()
@@ -590,14 +602,3 @@ def matepair_process():
 
 if __name__ == '__main__':
     matepair_process()
-    
-        
-
-            
-
-
-    
-
-    
-
-
